@@ -107,7 +107,7 @@ class PETSc(package.Package):
       petscconf_h = os.path.join(self.dir,'include','petscconf.h')
 
     self.buildsharedlib = False
-    self.bfort = 'nobfortinpetsc'
+    self.lib_name_suffix = ''
     try:
       with open(self.petscvariables) as f:
         for l in f.readlines():
@@ -125,7 +125,7 @@ class PETSc(package.Package):
           elif k == 'BUILDSHAREDLIB' and v=='yes':
             self.buildsharedlib = True
           else:
-            if k in ['AR','AR_FLAGS','AR_LIB_SUFFIX','BFORT','BLASLAPACK_LIB','CC','CC_FLAGS','CC_LINKER_SLFLAG','CMAKE','CONFIGURE_OPTIONS','CPP','CXX','CXX_FLAGS','FC_FLAGS','FC_VERSION','MAKE','MAKE_NP','PREFIXDIR','RANLIB','SCALAPACK_LIB','SEDINPLACE','SL_LINKER_SUFFIX']:
+            if k in ['AR','AR_FLAGS','AR_LIB_SUFFIX','BFORT','BLASLAPACK_LIB','CC','CC_FLAGS','CC_LINKER_SLFLAG','CMAKE','CONFIGURE_OPTIONS','CPP','CXX','CXX_FLAGS','FC_FLAGS','FC_VERSION','LIB_NAME_SUFFIX','MAKE','MAKE_NP','PREFIXDIR','RANLIB','SCALAPACK_LIB','SEDINPLACE','SL_LINKER_SUFFIX']:
               setattr(self,k.lower(),v)
     except:
       self.log.Exit('Cannot process file ' + self.petscvariables)
@@ -249,6 +249,15 @@ class PETSc(package.Package):
     except RuntimeError:
       pass
 
+  def isIntel(self):
+    '''returns true if the compiler is an Intel compiler'''
+    try:
+      (result, output) = self.RunCommand(self.cc+' --help | head -n 80',)
+      if 'Intel(R)' in output:
+        return 1
+    except RuntimeError:
+      pass
+
   def removeWarningFlags(self,flags):
     outflags = []
     for flag in flags:
