@@ -39,7 +39,7 @@ typedef struct _p_SVD* SVD;
 
 .seealso: SVDSetType(), SVD
 J*/
-typedef const char* SVDType;
+typedef const char *SVDType;
 #define SVDCROSS       "cross"
 #define SVDCYCLIC      "cyclic"
 #define SVDLAPACK      "lapack"
@@ -61,9 +61,9 @@ SLEPC_EXTERN PetscClassId SVD_CLASSID;
 
 .seealso: SVDSetProblemType(), SVDGetProblemType()
 E*/
-typedef enum { SVD_STANDARD=1,
-               SVD_GENERALIZED,   /* GSVD */
-               SVD_HYPERBOLIC     /* HSVD */
+typedef enum { SVD_STANDARD    = 1,
+               SVD_GENERALIZED = 2,  /* GSVD */
+               SVD_HYPERBOLIC  = 3   /* HSVD */
              } SVDProblemType;
 
 /*E
@@ -110,7 +110,8 @@ typedef enum { SVD_CONV_ABS,
 .seealso: SVDSetStoppingTest(), SVDSetStoppingTestFunction()
 E*/
 typedef enum { SVD_STOP_BASIC,
-               SVD_STOP_USER } SVDStop;
+               SVD_STOP_USER,
+               SVD_STOP_THRESHOLD } SVDStop;
 
 /*E
     SVDConvergedReason - Reason a singular value solver was said to
@@ -131,6 +132,23 @@ typedef enum {/* converged */
               SVD_CONVERGED_ITERATING          =  0 } SVDConvergedReason;
 SLEPC_EXTERN const char *const*SVDConvergedReasons;
 
+/*S
+   SVDStoppingCtx - Data structure (C struct) to hold additional information to
+   be used in some stopping test functions.
+
+   Level: advanced
+
+.seealso: SVDSetStoppingTestFunction()
+S*/
+struct _n_SVDStoppingCtx {
+  PetscReal firstsv;    /* the value of the first converged singular value */
+  PetscReal lastsv;     /* the value of the last converged singular value */
+  PetscReal thres;      /* threshold set with SVDSetThreshold() */
+  PetscBool threlative; /* threshold is relative */
+  SVDWhich  which;      /* which singular values are being computed */
+};
+typedef struct _n_SVDStoppingCtx* SVDStoppingCtx;
+
 SLEPC_EXTERN PetscErrorCode SVDCreate(MPI_Comm,SVD*);
 SLEPC_EXTERN PetscErrorCode SVDSetBV(SVD,BV,BV);
 SLEPC_EXTERN PetscErrorCode SVDGetBV(SVD,BV*,BV*);
@@ -159,6 +177,8 @@ SLEPC_EXTERN PetscErrorCode SVDSetTolerances(SVD,PetscReal,PetscInt);
 SLEPC_EXTERN PetscErrorCode SVDGetTolerances(SVD,PetscReal*,PetscInt*);
 SLEPC_EXTERN PetscErrorCode SVDSetWhichSingularTriplets(SVD,SVDWhich);
 SLEPC_EXTERN PetscErrorCode SVDGetWhichSingularTriplets(SVD,SVDWhich*);
+SLEPC_EXTERN PetscErrorCode SVDSetThreshold(SVD,PetscReal,PetscBool);
+SLEPC_EXTERN PetscErrorCode SVDGetThreshold(SVD,PetscReal*,PetscBool*);
 SLEPC_EXTERN PetscErrorCode SVDSetFromOptions(SVD);
 SLEPC_EXTERN PetscErrorCode SVDSetOptionsPrefix(SVD,const char*);
 SLEPC_EXTERN PetscErrorCode SVDAppendOptionsPrefix(SVD,const char*);
@@ -176,6 +196,7 @@ SLEPC_EXTERN PetscErrorCode SVDConvergedMaxIt(SVD,PetscReal,PetscReal,PetscReal*
 SLEPC_EXTERN PetscErrorCode SVDSetStoppingTest(SVD,SVDStop);
 SLEPC_EXTERN PetscErrorCode SVDGetStoppingTest(SVD,SVDStop*);
 SLEPC_EXTERN PetscErrorCode SVDStoppingBasic(SVD,PetscInt,PetscInt,PetscInt,PetscInt,SVDConvergedReason*,void*);
+SLEPC_EXTERN PetscErrorCode SVDStoppingThreshold(SVD,PetscInt,PetscInt,PetscInt,PetscInt,SVDConvergedReason*,void*);
 SLEPC_EXTERN PetscErrorCode SVDGetConvergedReason(SVD,SVDConvergedReason*);
 SLEPC_EXTERN PetscErrorCode SVDGetConverged(SVD,PetscInt*);
 SLEPC_EXTERN PetscErrorCode SVDGetSingularTriplet(SVD,PetscInt,PetscReal*,Vec,Vec);
@@ -202,22 +223,21 @@ SLEPC_EXTERN PetscErrorCode SVDSetTrackAll(SVD,PetscBool);
 SLEPC_EXTERN PetscErrorCode SVDGetTrackAll(SVD,PetscBool*);
 
 SLEPC_EXTERN PetscErrorCode SVDMonitor(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt);
-SLEPC_EXTERN PetscErrorCode SVDMonitorSet(SVD,PetscErrorCode (*)(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,void*),void*,PetscErrorCode (*)(void**));
+SLEPC_EXTERN PetscErrorCode SVDMonitorSet(SVD,PetscErrorCode (*)(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,void*),void*,PetscCtxDestroyFn*);
 SLEPC_EXTERN PetscErrorCode SVDMonitorCancel(SVD);
 SLEPC_EXTERN PetscErrorCode SVDGetMonitorContext(SVD,void*);
 
 SLEPC_EXTERN PetscErrorCode SVDMonitorSetFromOptions(SVD,const char[],const char[],void*,PetscBool);
-SLEPC_EXTERN PetscErrorCode SVDMonitorLGCreate(MPI_Comm,const char[],const char[],const char[],PetscInt,const char*[],int,int,int,int,PetscDrawLG*);
 SLEPC_EXTERN PetscErrorCode SVDMonitorFirst(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
 SLEPC_EXTERN PetscErrorCode SVDMonitorFirstDrawLG(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode SVDMonitorFirstDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode SVDMonitorFirstDrawLGCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode SVDMonitorAll(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
 SLEPC_EXTERN PetscErrorCode SVDMonitorAllDrawLG(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode SVDMonitorAllDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode SVDMonitorAllDrawLGCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode SVDMonitorConverged(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode SVDMonitorConvergedCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode SVDMonitorConvergedCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode SVDMonitorConvergedDrawLG(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode SVDMonitorConvergedDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode SVDMonitorConvergedDrawLGCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode SVDMonitorConvergedDestroy(PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode SVDMonitorConditioning(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*);
 
@@ -229,6 +249,7 @@ SLEPC_EXTERN PetscErrorCode SVDRegister(const char[],PetscErrorCode(*)(SVD));
 SLEPC_EXTERN PetscErrorCode SVDMonitorRegister(const char[],PetscViewerType,PetscViewerFormat,PetscErrorCode(*)(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscErrorCode(*)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**),PetscErrorCode(*)(PetscViewerAndFormat**));
 
 SLEPC_EXTERN PetscErrorCode SVDAllocateSolution(SVD,PetscInt);
+SLEPC_EXTERN PetscErrorCode SVDReallocateSolution(SVD,PetscInt);
 
 /*S
   SVDConvergenceTestFn - A prototype of an SVD convergence test function that would be passed to SVDSetConvergenceTestFunction()
@@ -266,8 +287,8 @@ PETSC_EXTERN_TYPEDEF typedef PetscErrorCode(SVDConvergenceTestFn)(SVD svd,PetscR
 S*/
 PETSC_EXTERN_TYPEDEF typedef PetscErrorCode(SVDStoppingTestFn)(SVD svd,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nsv,SVDConvergedReason *reason,void *ctx);
 
-SLEPC_EXTERN PetscErrorCode SVDSetConvergenceTestFunction(SVD,SVDConvergenceTestFn*,void*,PetscErrorCode (*)(void*));
-SLEPC_EXTERN PetscErrorCode SVDSetStoppingTestFunction(SVD,SVDStoppingTestFn*,void*,PetscErrorCode (*)(void*));
+SLEPC_EXTERN PetscErrorCode SVDSetConvergenceTestFunction(SVD,SVDConvergenceTestFn*,void*,PetscCtxDestroyFn*);
+SLEPC_EXTERN PetscErrorCode SVDSetStoppingTestFunction(SVD,SVDStoppingTestFn*,void*,PetscCtxDestroyFn*);
 
 /* --------- options specific to particular solvers -------- */
 
@@ -321,9 +342,9 @@ SLEPC_EXTERN PetscErrorCode SVDTRLanczosGetScale(SVD,PetscReal*);
 
 .seealso: SVDPRIMMESetMethod(), SVDPRIMMEGetMethod()
 E*/
-typedef enum { SVD_PRIMME_HYBRID=1,
-               SVD_PRIMME_NORMALEQUATIONS,
-               SVD_PRIMME_AUGMENTED } SVDPRIMMEMethod;
+typedef enum { SVD_PRIMME_HYBRID          = 1,
+               SVD_PRIMME_NORMALEQUATIONS = 2,
+               SVD_PRIMME_AUGMENTED       = 3 } SVDPRIMMEMethod;
 SLEPC_EXTERN const char *SVDPRIMMEMethods[];
 
 SLEPC_EXTERN PetscErrorCode SVDPRIMMESetBlockSize(SVD,PetscInt);
@@ -338,9 +359,9 @@ SLEPC_EXTERN PetscErrorCode SVDPRIMMEGetMethod(SVD,SVDPRIMMEMethod*);
 
 .seealso: SVDKSVDSetEigenMethod(), SVDKSVDGetEigenMethod()
 E*/
-typedef enum { SVD_KSVD_EIGEN_MRRR=1,
-               SVD_KSVD_EIGEN_DC,
-               SVD_KSVD_EIGEN_ELPA } SVDKSVDEigenMethod;
+typedef enum { SVD_KSVD_EIGEN_MRRR = 1,
+               SVD_KSVD_EIGEN_DC   = 2,
+               SVD_KSVD_EIGEN_ELPA = 3 } SVDKSVDEigenMethod;
 SLEPC_EXTERN const char *SVDKSVDEigenMethods[];
 
 /*E
@@ -350,8 +371,8 @@ SLEPC_EXTERN const char *SVDKSVDEigenMethods[];
 
 .seealso: SVDKSVDSetPolarMethod(), SVDKSVDGetPolarMethod()
 E*/
-typedef enum { SVD_KSVD_POLAR_QDWH=1,
-               SVD_KSVD_POLAR_ZOLOPD } SVDKSVDPolarMethod;
+typedef enum { SVD_KSVD_POLAR_QDWH   = 1,
+               SVD_KSVD_POLAR_ZOLOPD = 2 } SVDKSVDPolarMethod;
 SLEPC_EXTERN const char *SVDKSVDPolarMethods[];
 
 SLEPC_EXTERN PetscErrorCode SVDKSVDSetEigenMethod(SVD,SVDKSVDEigenMethod);
