@@ -13,17 +13,17 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      22
-#define SLEPC_VERSION_SUBMINOR   2
-#define SLEPC_RELEASE_DATE       "September 29, 2024"
-#define SLEPC_VERSION_DATE       "Dec 02, 2024"
+#define SLEPC_VERSION_MINOR      23
+#define SLEPC_VERSION_SUBMINOR   1
+#define SLEPC_RELEASE_DATE       "March 29, 2025"
+#define SLEPC_VERSION_DATE       "May 01, 2025"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "v3.22.2"
+#define SLEPC_VERSION_GIT        "v3.23.1"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2024-12-02 09:38:06 +0100"
+#define SLEPC_VERSION_DATE_GIT   "2025-05-01 12:17:35 +0200"
 #endif
 
 #define SLEPC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
