@@ -55,13 +55,14 @@ static PetscErrorCode EPSSetUp_ELPA(EPS eps)
   EPSCheckNotStructured(eps);
   PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&isshift));
   PetscCheck(isshift,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support spectral transformations");
+  if (eps->nev==0) eps->nev = 1;
   eps->ncv = eps->n;
   if (eps->mpd!=PETSC_DETERMINE) PetscCall(PetscInfo(eps,"Warning: parameter mpd ignored\n"));
   if (eps->max_it==PETSC_DETERMINE) eps->max_it = 1;
   if (!eps->which) PetscCall(EPSSetWhichEigenpairs_Default(eps));
   PetscCheck(eps->which!=EPS_ALL || eps->inta==eps->intb,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support interval computation");
-  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_STOPPING);
-  EPSCheckIgnored(eps,EPS_FEATURE_EXTRACTION | EPS_FEATURE_CONVERGENCE);
+  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION);
+  EPSCheckIgnored(eps,EPS_FEATURE_EXTRACTION | EPS_FEATURE_CONVERGENCE | EPS_FEATURE_STOPPING);
   PetscCall(EPSAllocateSolution(eps,0));
 
   /* convert matrices */
@@ -95,7 +96,7 @@ static PetscErrorCode EPSSolve_ELPA(EPS eps)
   PetscCall(MatDuplicate(A,MAT_DO_NOT_COPY_VALUES,&Q));
   q = (Mat_ScaLAPACK*)Q->data;
 
-  PetscCallELPARET(elpa_init,20200417);    /* 20171201 */
+  PetscCallELPARET(elpa_init,20250131);
   PetscCallELPANOARG(handle = elpa_allocate);
 
   /* set parameters of the matrix and its MPI distribution */
@@ -110,9 +111,9 @@ static PetscErrorCode EPSSolve_ELPA(EPS eps)
   if (B) PetscCallELPA(elpa_set,handle,"blacs_context",a->grid->ictxt);
 
   /* setup and set tunable run-time options */
-  PetscCallELPARET(elpa_setup,handle);
   PetscCallELPA(elpa_set,handle,"solver",ELPA_SOLVER_2STAGE);
   /* PetscCallELPA(elpa_print_settings,handle); */
+  PetscCallELPARET(elpa_setup,handle);
 
   /* solve the eigenvalue problem */
   if (B) {
