@@ -45,7 +45,7 @@ PetscErrorCode EPSMonitorSetFromOptions(EPS eps,const char opt[],const char name
   PetscBool            flg;
 
   PetscFunctionBegin;
-  PetscCall(PetscOptionsGetViewer(PetscObjectComm((PetscObject)eps),((PetscObject)eps)->options,((PetscObject)eps)->prefix,opt,&viewer,&format,&flg));
+  PetscCall(PetscOptionsCreateViewer(PetscObjectComm((PetscObject)eps),((PetscObject)eps)->options,((PetscObject)eps)->prefix,opt,&viewer,&format,&flg));
   if (!flg) PetscFunctionReturn(PETSC_SUCCESS);
 
   PetscCall(PetscViewerGetType(viewer,&vtype));
@@ -58,8 +58,8 @@ PetscErrorCode EPSMonitorSetFromOptions(EPS eps,const char opt[],const char name
   if (!dfunc) dfunc = PetscViewerAndFormatDestroy;
 
   PetscCall((*cfunc)(viewer,format,ctx,&vf));
-  PetscCall(PetscOptionsRestoreViewer(&viewer));
-  PetscCall(EPSMonitorSet(eps,mfunc,vf,(PetscErrorCode(*)(void **))dfunc));
+  PetscCall(PetscViewerDestroy(&viewer));
+  PetscCall(EPSMonitorSet(eps,mfunc,vf,(PetscCtxDestroyFn*)dfunc));
   if (trackall) PetscCall(EPSSetTrackAll(eps,PETSC_TRUE));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -108,8 +108,10 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
     if (flg) PetscCall(EPSSetProblemType(eps,EPS_GNHEP));
     PetscCall(PetscOptionsBoolGroup("-eps_pos_gen_non_hermitian","Generalized non-Hermitian eigenvalue problem with positive semi-definite B","EPSSetProblemType",&flg));
     if (flg) PetscCall(EPSSetProblemType(eps,EPS_PGNHEP));
-    PetscCall(PetscOptionsBoolGroupEnd("-eps_gen_indefinite","Generalized Hermitian-indefinite eigenvalue problem","EPSSetProblemType",&flg));
+    PetscCall(PetscOptionsBoolGroup("-eps_gen_indefinite","Generalized Hermitian-indefinite eigenvalue problem","EPSSetProblemType",&flg));
     if (flg) PetscCall(EPSSetProblemType(eps,EPS_GHIEP));
+    PetscCall(PetscOptionsBoolGroupEnd("-eps_bse","Structured Bethe-Salpeter eigenvalue problem","EPSSetProblemType",&flg));
+    if (flg) PetscCall(EPSSetProblemType(eps,EPS_BSE));
 
     PetscCall(PetscOptionsBoolGroupBegin("-eps_ritz","Rayleigh-Ritz extraction","EPSSetExtraction",&flg));
     if (flg) PetscCall(EPSSetExtraction(eps,EPS_RITZ));
@@ -140,6 +142,12 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
     PetscCall(PetscOptionsReal("-eps_tol","Tolerance","EPSSetTolerances",SlepcDefaultTol(eps->tol),&r,&flg2));
     if (flg1 || flg2) PetscCall(EPSSetTolerances(eps,r,i));
 
+    r = eps->thres;
+    PetscCall(PetscOptionsReal("-eps_threshold_absolute","Absolute threshold","EPSSetThreshold",r,&r,&flg));
+    if (flg) PetscCall(EPSSetThreshold(eps,r,PETSC_FALSE));
+    PetscCall(PetscOptionsReal("-eps_threshold_relative","Relative threshold","EPSSetThreshold",r,&r,&flg));
+    if (flg) PetscCall(EPSSetThreshold(eps,r,PETSC_TRUE));
+
     PetscCall(PetscOptionsBoolGroupBegin("-eps_conv_rel","Relative error convergence test","EPSSetConvergenceTest",&flg));
     if (flg) PetscCall(EPSSetConvergenceTest(eps,EPS_CONV_REL));
     PetscCall(PetscOptionsBoolGroup("-eps_conv_norm","Convergence test relative to the eigenvalue and the matrix norms","EPSSetConvergenceTest",&flg));
@@ -151,11 +159,14 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
 
     PetscCall(PetscOptionsBoolGroupBegin("-eps_stop_basic","Stop iteration if all eigenvalues converged or max_it reached","EPSSetStoppingTest",&flg));
     if (flg) PetscCall(EPSSetStoppingTest(eps,EPS_STOP_BASIC));
+    PetscCall(PetscOptionsBoolGroup("-eps_stop_threshold","Stop iteration if a converged eigenvalue is below/above the threshold","EPSSetStoppingTest",&flg));
+    if (flg) PetscCall(EPSSetStoppingTest(eps,EPS_STOP_THRESHOLD));
     PetscCall(PetscOptionsBoolGroupEnd("-eps_stop_user","User-defined stopping test","EPSSetStoppingTest",&flg));
     if (flg) PetscCall(EPSSetStoppingTest(eps,EPS_STOP_USER));
 
     i = eps->nev;
     PetscCall(PetscOptionsInt("-eps_nev","Number of eigenvalues to compute","EPSSetDimensions",eps->nev,&i,&flg1));
+    if (!flg1) i = PETSC_CURRENT;
     j = eps->ncv;
     PetscCall(PetscOptionsInt("-eps_ncv","Number of basis vectors","EPSSetDimensions",eps->ncv,&j,&flg2));
     k = eps->mpd;
@@ -241,7 +252,7 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    EPSGetTolerances - Gets the tolerance and maximum iteration count used
    by the EPS convergence tests.
 
@@ -286,7 +297,10 @@ PetscErrorCode EPSGetTolerances(EPS eps,PetscReal *tol,PetscInt *maxits)
 -  -eps_max_it <maxits> - Sets the maximum number of iterations allowed
 
    Notes:
-   Use PETSC_DEFAULT for either argument to assign a reasonably good value.
+   Use PETSC_CURRENT to retain the current value of any of the parameters.
+   Use PETSC_DETERMINE for either argument to assign a default value computed
+   internally (may be different in each solver).
+   For maxits use PETSC_UMLIMITED to indicate there is no upper bound on this value.
 
    Level: intermediate
 
@@ -298,24 +312,26 @@ PetscErrorCode EPSSetTolerances(EPS eps,PetscReal tol,PetscInt maxits)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveReal(eps,tol,2);
   PetscValidLogicalCollectiveInt(eps,maxits,3);
-  if (tol == (PetscReal)PETSC_DEFAULT) {
-    eps->tol   = PETSC_DEFAULT;
+  if (tol == (PetscReal)PETSC_DETERMINE) {
+    eps->tol   = PETSC_DETERMINE;
     eps->state = EPS_STATE_INITIAL;
-  } else {
+  } else if (tol != (PetscReal)PETSC_CURRENT) {
     PetscCheck(tol>0.0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
     eps->tol = tol;
   }
-  if (maxits == PETSC_DEFAULT || maxits == PETSC_DECIDE) {
-    eps->max_it = PETSC_DEFAULT;
+  if (maxits == PETSC_DETERMINE) {
+    eps->max_it = PETSC_DETERMINE;
     eps->state  = EPS_STATE_INITIAL;
-  } else {
+  } else if (maxits == PETSC_UNLIMITED) {
+    eps->max_it = PETSC_INT_MAX;
+  } else if (maxits != PETSC_CURRENT) {
     PetscCheck(maxits>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
     eps->max_it = maxits;
   }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    EPSGetDimensions - Gets the number of eigenvalues to compute
    and the dimension of the subspace.
 
@@ -337,7 +353,7 @@ PetscErrorCode EPSGetDimensions(EPS eps,PetscInt *nev,PetscInt *ncv,PetscInt *mp
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (nev) *nev = eps->nev;
+  if (nev) *nev = eps->nev? eps->nev: 1;
   if (ncv) *ncv = eps->ncv;
   if (mpd) *mpd = eps->mpd;
   PetscFunctionReturn(PETSC_SUCCESS);
@@ -361,8 +377,9 @@ PetscErrorCode EPSGetDimensions(EPS eps,PetscInt *nev,PetscInt *ncv,PetscInt *mp
 -  -eps_mpd <mpd> - Sets the maximum projected dimension
 
    Notes:
-   Use PETSC_DEFAULT for ncv and mpd to assign a reasonably good value, which is
-   dependent on the solution method.
+   Use PETSC_DETERMINE for ncv and mpd to assign a reasonably good value, which is
+   dependent on the solution method. For any of the arguments, use PETSC_CURRENT
+   to preserve the current value.
 
    The parameters ncv and mpd are intimately related, so that the user is advised
    to set one of them at most. Normal usage is that
@@ -388,17 +405,19 @@ PetscErrorCode EPSSetDimensions(EPS eps,PetscInt nev,PetscInt ncv,PetscInt mpd)
   PetscValidLogicalCollectiveInt(eps,nev,2);
   PetscValidLogicalCollectiveInt(eps,ncv,3);
   PetscValidLogicalCollectiveInt(eps,mpd,4);
-  PetscCheck(nev>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
-  eps->nev = nev;
-  if (ncv == PETSC_DECIDE || ncv == PETSC_DEFAULT) {
-    eps->ncv = PETSC_DEFAULT;
-  } else {
+  if (nev != PETSC_CURRENT) {
+    PetscCheck(nev>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
+    eps->nev = nev;
+  }
+  if (ncv == PETSC_DETERMINE) {
+    eps->ncv = PETSC_DETERMINE;
+  } else if (ncv != PETSC_CURRENT) {
     PetscCheck(ncv>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
     eps->ncv = ncv;
   }
-  if (mpd == PETSC_DECIDE || mpd == PETSC_DEFAULT) {
-    eps->mpd = PETSC_DEFAULT;
-  } else {
+  if (mpd == PETSC_DETERMINE) {
+    eps->mpd = PETSC_DETERMINE;
+  } else if (mpd != PETSC_CURRENT) {
     PetscCheck(mpd>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
     eps->mpd = mpd;
   }
@@ -527,6 +546,111 @@ PetscErrorCode EPSGetWhichEigenpairs(EPS eps,EPSWhich *which)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
+/*@
+   EPSSetThreshold - Sets the threshold used in the threshold stopping test.
+
+   Logically Collective
+
+   Input Parameters:
++  eps   - the eigenvalue solver context
+.  thres - the threshold value
+-  rel   - whether the threshold is relative or not
+
+   Options Database Keys:
++  -eps_threshold_absolute <thres> - Sets an absolute threshold
+-  -eps_threshold_relative <thres> - Sets a relative threshold
+
+   Notes:
+   This function internally calls EPSSetStoppingTest() to set a special stopping
+   test based on the threshold, where eigenvalues are computed in sequence until
+   one of the computed eigenvalues is below the threshold (in magnitude). This is
+   the interpretation in case of searching for largest eigenvalues in magnitude,
+   see EPSSetWhichEigenpairs().
+
+   If the solver is configured to compute smallest magnitude eigenvalues, then the
+   threshold must be interpreted in the opposite direction, i.e., the computation
+   will stop when one of the computed values is above the threshold (in magnitude).
+
+   The threshold can also be used when computing largest/smallest real eigenvalues
+   (i.e, rightmost or leftmost), in which case the threshold is allowed to be
+   negative. The solver will stop when one of the computed eigenvalues is above
+   or below the threshold (considering the real part of the eigenvalue). This mode
+   is allowed only in problem types whose eigenvalues are always real (e.g., HEP).
+
+   In the case of largest magnitude eigenvalues, the threshold can be made relative
+   with respect to the dominant eigenvalue. Otherwise, the argument rel should be
+   PETSC_FALSE.
+
+   An additional use case is with target magnitude selection of eigenvalues (e.g.,
+   with shift-and-invert), but this must be used with caution to avoid unexpected
+   behaviour. With an absolute threshold, the solver will assume that leftmost
+   eigenvalues are being computed (e.g., with target=0 for a problem with real
+   positive eigenvalues). In case of a relative threshold, a value of threshold<1
+   implies that the wanted eigenvalues are the largest ones, and otherwise the
+   solver assumes that smallest eigenvalues are being computed.
+
+   The test against the threshold is done for converged eigenvalues, which
+   implies that the final number of converged eigenvalues will be at least
+   one more than the actual number of values below/above the threshold.
+
+   Since the number of computed eigenvalues is not known a priori, the solver
+   will need to reallocate the basis of vectors internally, to have enough room
+   to accommodate all the eigenvectors. Hence, this option must be used with
+   caution to avoid out-of-memory problems. The recommendation is to set the value
+   of ncv to be larger than the estimated number of eigenvalues, to minimize the
+   number of reallocations.
+
+   If a number of wanted eigenvalues has been set with EPSSetDimensions()
+   it is also taken into account and the solver will stop when one of the two
+   conditions (threshold or number of converged values) is met.
+
+   Use EPSSetStoppingTest() to return to the usual computation of a fixed number
+   of eigenvalues.
+
+   Level: advanced
+
+.seealso: EPSGetThreshold(), EPSSetStoppingTest(), EPSSetDimensions(), EPSSetWhichEigenpairs(), EPSSetProblemType()
+@*/
+PetscErrorCode EPSSetThreshold(EPS eps,PetscReal thres,PetscBool rel)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveReal(eps,thres,2);
+  PetscValidLogicalCollectiveBool(eps,rel,3);
+  if (eps->thres != thres || eps->threlative != rel) {
+    eps->thres = thres;
+    eps->threlative = rel;
+    eps->state = EPS_STATE_INITIAL;
+    PetscCall(EPSSetStoppingTest(eps,EPS_STOP_THRESHOLD));
+  }
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
+/*@
+   EPSGetThreshold - Gets the threshold used by the threshold stopping test.
+
+   Not Collective
+
+   Input Parameter:
+.  eps - the eigenvalue solver context
+
+   Output Parameters:
++  thres - the threshold
+-  rel   - whether the threshold is relative or not
+
+   Level: advanced
+
+.seealso: EPSSetThreshold()
+@*/
+PetscErrorCode EPSGetThreshold(EPS eps,PetscReal *thres,PetscBool *rel)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  if (thres) *thres = eps->thres;
+  if (rel)   *rel   = eps->threlative;
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
 /*@C
    EPSSetEigenvalueComparison - Specifies the eigenvalue comparison function
    when EPSSetWhichEigenpairs() is set to EPS_WHICH_USER.
@@ -535,18 +659,9 @@ PetscErrorCode EPSGetWhichEigenpairs(EPS eps,EPSWhich *which)
 
    Input Parameters:
 +  eps  - eigensolver context obtained from EPSCreate()
-.  func - a pointer to the comparison function
+.  func - the comparison function, see EPSEigenvalueComparisonFn for the calling sequence
 -  ctx  - a context pointer (the last parameter to the comparison function)
 
-   Calling sequence of func:
-$  PetscErrorCode func(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *res,void *ctx)
-+   ar     - real part of the 1st eigenvalue
-.   ai     - imaginary part of the 1st eigenvalue
-.   br     - real part of the 2nd eigenvalue
-.   bi     - imaginary part of the 2nd eigenvalue
-.   res    - result of comparison
--   ctx    - optional context, as set by EPSSetEigenvalueComparison()
-
    Note:
    The returning parameter 'res' can be
 +  negative - if the 1st eigenvalue is preferred to the 2st one
@@ -557,7 +672,7 @@ $  PetscErrorCode func(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar
 
 .seealso: EPSSetWhichEigenpairs(), EPSWhich
 @*/
-PetscErrorCode EPSSetEigenvalueComparison(EPS eps,PetscErrorCode (*func)(PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *res,void *ctx),void* ctx)
+PetscErrorCode EPSSetEigenvalueComparison(EPS eps,SlepcEigenvalueComparisonFn *func,void* ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -576,18 +691,8 @@ PetscErrorCode EPSSetEigenvalueComparison(EPS eps,PetscErrorCode (*func)(PetscSc
 
    Input Parameters:
 +  eps  - eigensolver context obtained from EPSCreate()
-.  func - a pointer to the evaluation function
--  ctx  - a context pointer (the last parameter to the evaluation function)
-
-   Calling sequence of func:
-$  PetscErrorCode func(PetscScalar er,PetscScalar ei,Vec xr,Vec xi,PetscScalar *rr,PetscScalar *ri,void *ctx)
-+   er     - real part of the current eigenvalue approximation
-.   ei     - imaginary part of the current eigenvalue approximation
-.   xr     - real part of the current eigenvector approximation
-.   xi     - imaginary part of the current eigenvector approximation
-.   rr     - result of evaluation (real part)
-.   ri     - result of evaluation (imaginary part)
--   ctx    - optional context, as set by EPSSetArbitrarySelection()
+.  func - the arbitrary selection function, see SlepcArbitrarySelectionFn for a calling sequence
+-  ctx  - a context pointer (the last parameter to the arbitrary selection function)
 
    Notes:
    This provides a mechanism to select eigenpairs by evaluating a user-defined
@@ -613,7 +718,7 @@ $  PetscErrorCode func(PetscScalar er,PetscScalar ei,Vec xr,Vec xi,PetscScalar *
 
 .seealso: EPSSetWhichEigenpairs()
 @*/
-PetscErrorCode EPSSetArbitrarySelection(EPS eps,PetscErrorCode (*func)(PetscScalar er,PetscScalar ei,Vec xr,Vec xi,PetscScalar *rr,PetscScalar *ri,void *ctx),void* ctx)
+PetscErrorCode EPSSetArbitrarySelection(EPS eps,SlepcArbitrarySelectionFn *func,void* ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
@@ -631,18 +736,9 @@ PetscErrorCode EPSSetArbitrarySelection(EPS eps,PetscErrorCode (*func)(PetscScal
 
    Input Parameters:
 +  eps     - eigensolver context obtained from EPSCreate()
-.  func    - a pointer to the convergence test function
-.  ctx     - context for private data for the convergence routine (may be null)
--  destroy - a routine for destroying the context (may be null)
-
-   Calling sequence of func:
-$  PetscErrorCode func(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx)
-+   eps    - eigensolver context obtained from EPSCreate()
-.   eigr   - real part of the eigenvalue
-.   eigi   - imaginary part of the eigenvalue
-.   res    - residual norm associated to the eigenpair
-.   errest - (output) computed error estimate
--   ctx    - optional context, as set by EPSSetConvergenceTestFunction()
+.  func    - convergence test function, see EPSConvergenceTestFn for the calling sequence
+.  ctx     - context for private data for the convergence routine (may be NULL)
+-  destroy - a routine for destroying the context (may be NULL), see PetscCtxDestroyFn for the calling sequence
 
    Note:
    If the error estimate returned by the convergence test function is less than
@@ -652,11 +748,11 @@ $  PetscErrorCode func(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,P
 
 .seealso: EPSSetConvergenceTest(), EPSSetTolerances()
 @*/
-PetscErrorCode EPSSetConvergenceTestFunction(EPS eps,PetscErrorCode (*func)(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx),void* ctx,PetscErrorCode (*destroy)(void*))
+PetscErrorCode EPSSetConvergenceTestFunction(EPS eps,EPSConvergenceTestFn *func,void* ctx,PetscCtxDestroyFn *destroy)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (eps->convergeddestroy) PetscCall((*eps->convergeddestroy)(eps->convergedctx));
+  if (eps->convergeddestroy) PetscCall((*eps->convergeddestroy)(&eps->convergedctx));
   eps->convergeduser    = func;
   eps->convergeddestroy = destroy;
   eps->convergedctx     = ctx;
@@ -750,19 +846,9 @@ PetscErrorCode EPSGetConvergenceTest(EPS eps,EPSConv *conv)
 
    Input Parameters:
 +  eps     - eigensolver context obtained from EPSCreate()
-.  func    - pointer to the stopping test function
-.  ctx     - context for private data for the stopping routine (may be null)
--  destroy - a routine for destroying the context (may be null)
-
-   Calling sequence of func:
-$  PetscErrorCode func(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,EPSConvergedReason *reason,void *ctx)
-+   eps    - eigensolver context obtained from EPSCreate()
-.   its    - current number of iterations
-.   max_it - maximum number of iterations
-.   nconv  - number of currently converged eigenpairs
-.   nev    - number of requested eigenpairs
-.   reason - (output) result of the stopping test
--   ctx    - optional context, as set by EPSSetStoppingTestFunction()
+.  func    - stopping test function, see EPSStoppingTestFn for the calling sequence
+.  ctx     - context for private data for the stopping routine (may be NULL)
+-  destroy - a routine for destroying the context (may be NULL), see PetscCtxDestroyFn for the calling sequence
 
    Note:
    Normal usage is to first call the default routine EPSStoppingBasic() and then
@@ -774,15 +860,16 @@ $  PetscErrorCode func(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,Petsc
 
 .seealso: EPSSetStoppingTest(), EPSStoppingBasic()
 @*/
-PetscErrorCode EPSSetStoppingTestFunction(EPS eps,PetscErrorCode (*func)(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,EPSConvergedReason *reason,void *ctx),void* ctx,PetscErrorCode (*destroy)(void*))
+PetscErrorCode EPSSetStoppingTestFunction(EPS eps,EPSStoppingTestFn *func,void* ctx,PetscCtxDestroyFn *destroy)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (eps->stoppingdestroy) PetscCall((*eps->stoppingdestroy)(eps->stoppingctx));
+  if (eps->stoppingdestroy) PetscCall((*eps->stoppingdestroy)(&eps->stoppingctx));
   eps->stoppinguser    = func;
   eps->stoppingdestroy = destroy;
   eps->stoppingctx     = ctx;
-  if (func == EPSStoppingBasic) eps->stop = EPS_STOP_BASIC;
+  if (func == EPSStoppingBasic) PetscCall(EPSSetStoppingTest(eps,EPS_STOP_BASIC));
+  else if (func == EPSStoppingThreshold) PetscCall(EPSSetStoppingTest(eps,EPS_STOP_THRESHOLD));
   else {
     eps->stop     = EPS_STOP_USER;
     eps->stopping = eps->stoppinguser;
@@ -801,13 +888,15 @@ PetscErrorCode EPSSetStoppingTestFunction(EPS eps,PetscErrorCode (*func)(EPS eps
 -  stop - the type of stopping test
 
    Options Database Keys:
-+  -eps_stop_basic - Sets the default stopping test
--  -eps_stop_user  - Selects the user-defined stopping test
++  -eps_stop_basic     - Sets the default stopping test
+.  -eps_stop_threshold - Sets the threshold stopping test
+-  -eps_stop_user      - Selects the user-defined stopping test
 
    Note:
    The parameter 'stop' can have one of these values
-+     EPS_STOP_BASIC - default stopping test
--     EPS_STOP_USER  - function set by EPSSetStoppingTestFunction()
++     EPS_STOP_BASIC     - default stopping test
+.     EPS_STOP_THRESHOLD - threshold stopping test)
+-     EPS_STOP_USER      - function set by EPSSetStoppingTestFunction()
 
    Level: advanced
 
@@ -820,6 +909,7 @@ PetscErrorCode EPSSetStoppingTest(EPS eps,EPSStop stop)
   PetscValidLogicalCollectiveEnum(eps,stop,2);
   switch (stop) {
     case EPS_STOP_BASIC: eps->stopping = EPSStoppingBasic; break;
+    case EPS_STOP_THRESHOLD: eps->stopping = EPSStoppingThreshold; break;
     case EPS_STOP_USER:
       PetscCheck(eps->stoppinguser,PetscObjectComm((PetscObject)eps),PETSC_ERR_ORDER,"Must call EPSSetStoppingTestFunction() first");
       eps->stopping = eps->stoppinguser;
@@ -872,21 +962,25 @@ PetscErrorCode EPSGetStoppingTest(EPS eps,EPSStop *stop)
 .  -eps_gen_non_hermitian - generalized non-Hermitian eigenvalue problem
 .  -eps_pos_gen_non_hermitian - generalized non-Hermitian eigenvalue problem
    with positive semi-definite B
--  -eps_gen_indefinite - generalized Hermitian-indefinite eigenvalue problem
+.  -eps_gen_indefinite - generalized Hermitian-indefinite eigenvalue problem
+-  -eps_bse - structured Bethe-Salpeter eigenvalue problem
 
    Notes:
-   Allowed values for the problem type are Hermitian (EPS_HEP), non-Hermitian
-   (EPS_NHEP), generalized Hermitian (EPS_GHEP), generalized non-Hermitian
-   (EPS_GNHEP), generalized non-Hermitian with positive semi-definite B
-   (EPS_PGNHEP), and generalized Hermitian-indefinite (EPS_GHIEP).
-
-   This function must be used to instruct SLEPc to exploit symmetry. If no
+   This function must be used to instruct SLEPc to exploit symmetry or other
+   kind of structure. If no
    problem type is specified, by default a non-Hermitian problem is assumed
    (either standard or generalized). If the user knows that the problem is
    Hermitian (i.e. A=A^H) or generalized Hermitian (i.e. A=A^H, B=B^H, and
    B positive definite) then it is recommended to set the problem type so
    that eigensolver can exploit these properties.
 
+   If the user does not call this function, the solver will use a reasonable
+   guess.
+
+   For structured problem types such as EPS_BSE, the matrices passed in via
+   EPSSetOperators() must have been created with the corresponding helper
+   function, i.e., MatCreateBSE().
+
    Level: intermediate
 
 .seealso: EPSSetOperators(), EPSSetType(), EPSGetProblemType(), EPSProblemType
@@ -902,31 +996,43 @@ PetscErrorCode EPSSetProblemType(EPS eps,EPSProblemType type)
       eps->isgeneralized = PETSC_FALSE;
       eps->ishermitian = PETSC_TRUE;
       eps->ispositive = PETSC_FALSE;
+      eps->isstructured = PETSC_FALSE;
       break;
     case EPS_NHEP:
       eps->isgeneralized = PETSC_FALSE;
       eps->ishermitian = PETSC_FALSE;
       eps->ispositive = PETSC_FALSE;
+      eps->isstructured = PETSC_FALSE;
       break;
     case EPS_GHEP:
       eps->isgeneralized = PETSC_TRUE;
       eps->ishermitian = PETSC_TRUE;
       eps->ispositive = PETSC_TRUE;
+      eps->isstructured = PETSC_FALSE;
       break;
     case EPS_GNHEP:
       eps->isgeneralized = PETSC_TRUE;
       eps->ishermitian = PETSC_FALSE;
       eps->ispositive = PETSC_FALSE;
+      eps->isstructured = PETSC_FALSE;
       break;
     case EPS_PGNHEP:
       eps->isgeneralized = PETSC_TRUE;
       eps->ishermitian = PETSC_FALSE;
       eps->ispositive = PETSC_TRUE;
+      eps->isstructured = PETSC_FALSE;
       break;
     case EPS_GHIEP:
       eps->isgeneralized = PETSC_TRUE;
       eps->ishermitian = PETSC_TRUE;
       eps->ispositive = PETSC_FALSE;
+      eps->isstructured = PETSC_FALSE;
+      break;
+    case EPS_BSE:
+      eps->isgeneralized = PETSC_FALSE;
+      eps->ishermitian = PETSC_FALSE;
+      eps->ispositive = PETSC_FALSE;
+      eps->isstructured = PETSC_TRUE;
       break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
@@ -998,7 +1104,10 @@ PetscErrorCode EPSSetExtraction(EPS eps,EPSExtraction extr)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveEnum(eps,extr,2);
-  eps->extraction = extr;
+  if (eps->extraction != extr) {
+    eps->state      = EPS_STATE_INITIAL;
+    eps->extraction = extr;
+  }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -1058,8 +1167,8 @@ PetscErrorCode EPSGetExtraction(EPS eps,EPSExtraction *extr)
    matrices to have the MatMultTranspose operation defined.
 
    The parameter 'its' is the number of iterations performed by the method. The
-   cutoff value is used only in the two-side variant. Use PETSC_DEFAULT to assign
-   a reasonably good value.
+   cutoff value is used only in the two-side variant. Use PETSC_DETERMINE to assign
+   a reasonably good value, or PETSC_CURRENT to leave the value unchanged.
 
    User-defined balancing is allowed provided that the corresponding matrix
    is set via STSetBalanceMatrix.
@@ -1088,20 +1197,20 @@ PetscErrorCode EPSSetBalance(EPS eps,EPSBalance bal,PetscInt its,PetscReal cutof
     default:
       SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid value of argument 'bal'");
   }
-  if (its==PETSC_DECIDE || its==PETSC_DEFAULT) eps->balance_its = 5;
-  else {
+  if (its==PETSC_DETERMINE) eps->balance_its = 5;
+  else if (its!=PETSC_CURRENT) {
     PetscCheck(its>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of its. Must be > 0");
     eps->balance_its = its;
   }
-  if (cutoff==(PetscReal)PETSC_DECIDE || cutoff==(PetscReal)PETSC_DEFAULT) eps->balance_cutoff = 1e-8;
-  else {
+  if (cutoff==(PetscReal)PETSC_DETERMINE) eps->balance_cutoff = 1e-8;
+  else if (cutoff!=(PetscReal)PETSC_CURRENT) {
     PetscCheck(cutoff>0.0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of cutoff. Must be > 0");
     eps->balance_cutoff = cutoff;
   }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    EPSGetBalance - Gets the balancing type used by the EPS object, and the
    associated parameters.
 
@@ -1370,7 +1479,7 @@ PetscErrorCode EPSGetPurify(EPS eps,PetscBool *purify)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    EPSSetOptionsPrefix - Sets the prefix used for searching for all
    EPS options in the database.
 
@@ -1412,7 +1521,7 @@ PetscErrorCode EPSSetOptionsPrefix(EPS eps,const char *prefix)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    EPSAppendOptionsPrefix - Appends to the prefix used for searching for all
    EPS options in the database.
 
@@ -1446,7 +1555,7 @@ PetscErrorCode EPSAppendOptionsPrefix(EPS eps,const char *prefix)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    EPSGetOptionsPrefix - Gets the prefix used for searching for all
    EPS options in the database.
 
