@@ -28,7 +28,7 @@ int main(int argc,char **argv)
   LMEConvergedReason reason;
 
   PetscFunctionBeginUser;
-  PetscCall(SlepcInitialize(&argc,&argv,(char*)0,help));
+  PetscCall(SlepcInitialize(&argc,&argv,NULL,help));
 
   PetscCall(PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL));
   PetscCall(PetscOptionsGetInt(NULL,NULL,"-m",&m,&flag));
@@ -115,7 +115,7 @@ int main(int argc,char **argv)
   /*
      (Optional) Set other solver options
   */
-  PetscCall(LMESetTolerances(lme,1e-07,PETSC_DEFAULT));
+  PetscCall(LMESetTolerances(lme,1e-07,PETSC_CURRENT));
 
   /*
      Set solver parameters at runtime
