@@ -97,7 +97,7 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
   BVCheckSizes(Y,2);
   PetscValidType(M,3);
   PetscCheckSameTypeAndComm(X,1,Y,2);
-  PetscCheckTypeNames(M,MATSEQDENSE,MATSEQDENSECUDA);
+  SlepcMatCheckSeq(M);
 
   PetscCall(MatGetSize(M,&m,&n));
   PetscCheck(m>=Y->k,PetscObjectComm((PetscObject)X),PETSC_ERR_ARG_SIZ,"Mat argument has %" PetscInt_FMT " rows, should have at least %" PetscInt_FMT,m,Y->k);
@@ -134,7 +134,7 @@ PetscErrorCode BVDot(BV X,BV Y,Mat M)
 
    Notes:
    This is analogue to VecMDot(), but using BV to represent a collection
-   of vectors. The result is m = X^H*y, so m_i is equal to x_j^H y. Note
+   of vectors. The result is m = X^H*y, so m_i is equal to x_i^H y. Note
    that here X is transposed as opposed to BVDot().
 
    If a non-standard inner product has been specified with BVSetMatrix(),
@@ -500,7 +500,7 @@ PetscErrorCode BVNorm(BV bv,NormType type,PetscReal *val)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveEnum(bv,type,2);
-  PetscValidRealPointer(val,3);
+  PetscAssertPointer(val,3);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -544,7 +544,7 @@ PetscErrorCode BVNormVec(BV bv,Vec v,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidHeaderSpecific(v,VEC_CLASSID,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidRealPointer(val,4);
+  PetscAssertPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
   PetscValidType(v,2);
@@ -586,7 +586,7 @@ PetscErrorCode BVNormVecBegin(BV bv,Vec v,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidHeaderSpecific(v,VEC_CLASSID,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidRealPointer(val,4);
+  PetscAssertPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
   PetscValidType(v,2);
@@ -625,7 +625,7 @@ PetscErrorCode BVNormVecEnd(BV bv,Vec v,NormType type,PetscReal *val)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidRealPointer(val,4);
+  PetscAssertPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -667,7 +667,7 @@ PetscErrorCode BVNormColumn(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidRealPointer(val,4);
+  PetscAssertPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -711,7 +711,7 @@ PetscErrorCode BVNormColumnBegin(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidRealPointer(val,4);
+  PetscAssertPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -766,7 +766,7 @@ PetscErrorCode BVNormColumnEnd(BV bv,PetscInt j,NormType type,PetscReal *val)
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(bv,j,2);
   PetscValidLogicalCollectiveEnum(bv,type,3);
-  PetscValidRealPointer(val,4);
+  PetscAssertPointer(val,4);
   PetscValidType(bv,1);
   BVCheckSizes(bv,1);
 
@@ -1113,7 +1113,7 @@ PetscErrorCode BVMatProject(BV X,Mat A,BV Y,Mat M)
   BVCheckSizes(Y,3);
   PetscValidType(M,4);
   PetscCheckSameTypeAndComm(X,1,Y,3);
-  PetscCheckTypeNames(M,MATSEQDENSE,MATSEQDENSECUDA);
+  SlepcMatCheckSeq(M);
 
   PetscCall(MatGetSize(M,&m,&n));
   PetscCall(MatDenseGetLDA(M,&ldm));
