@@ -63,6 +63,36 @@ PetscErrorCode DSAllocateMat_Private(DS ds,DSMatType m)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
+PetscErrorCode DSReallocateMat_Private(DS ds,DSMatType mat,PetscInt ld)
+{
+  Mat               M;
+  PetscInt          i,m,n,rows,cols;
+  PetscScalar       *dst;
+  PetscReal         *rdst;
+  const PetscScalar *src;
+  const PetscReal   *rsrc;
+
+  PetscFunctionBegin;
+  PetscCall(MatGetSize(ds->omat[mat],&m,&n));
+  rows = ld;
+  cols = m==n? ld: n;
+  PetscCall(MatCreateSeqDense(PETSC_COMM_SELF,rows,cols,NULL,&M));
+  PetscCall(MatDenseGetArrayRead(ds->omat[mat],&src));
+  PetscCall(MatDenseGetArray(M,&dst));
+  if (mat==DS_MAT_T && PetscDefined(USE_COMPLEX)) {
+    rsrc = (const PetscReal*)src;
+    rdst = (PetscReal*)dst;
+    for (i=0;i<3;i++) PetscCall(PetscArraycpy(rdst+i*ld,rsrc+i*ds->ld,ds->ld));
+  } else {
+    for (i=0;i<n;i++) PetscCall(PetscArraycpy(dst+i*ld,src+i*ds->ld,ds->ld));
+  }
+  PetscCall(MatDenseRestoreArrayRead(ds->omat[mat],&src));
+  PetscCall(MatDenseRestoreArray(M,&dst));
+  PetscCall(MatDestroy(&ds->omat[mat]));
+  ds->omat[mat] = M;
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
 PetscErrorCode DSAllocateWork_Private(DS ds,PetscInt s,PetscInt r,PetscInt i)
 {
   PetscFunctionBegin;
@@ -84,7 +114,7 @@ PetscErrorCode DSAllocateWork_Private(DS ds,PetscInt s,PetscInt r,PetscInt i)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    DSViewMat - Prints one of the internal DS matrices.
 
    Collective
@@ -395,7 +425,7 @@ PetscErrorCode DSSetIdentity(DS ds,DSMatType mat)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    DSOrthogonalize - Orthogonalize the columns of a matrix.
 
    Logically Collective
@@ -470,7 +500,7 @@ PetscErrorCode DSOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscInt *lindc
 static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar b,PetscScalar a,const PetscScalar *A,PetscInt _ldA,PetscInt rA,PetscInt cA,PetscBool At,const PetscScalar *B,PetscInt _ldB,PetscInt rB,PetscInt cB,PetscBool Bt)
 {
   PetscInt       tmp;
-  PetscBLASInt   m, n, k, ldA = _ldA, ldB = _ldB, ldC = _ldC;
+  PetscBLASInt   m, n, k, ldA, ldB, ldC;
   const char     *N = "N", *T = "C", *qA = N, *qB = N;
 
   PetscFunctionBegin;
@@ -478,6 +508,9 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
   PetscAssertPointer(C,1);
   PetscAssertPointer(A,5);
   PetscAssertPointer(B,10);
+  PetscCall(PetscBLASIntCast(_ldA,&ldA));
+  PetscCall(PetscBLASIntCast(_ldB,&ldB));
+  PetscCall(PetscBLASIntCast(_ldC,&ldC));
 
   /* Transpose if needed */
   if (At) tmp = rA, rA = cA, cA = tmp, qA = T;
@@ -494,7 +527,9 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
     else *C = PetscConj(*A) * PetscConj(*B);
     m = n = k = 1;
   } else {
-    m = rA; n = cB; k = cA;
+    PetscCall(PetscBLASIntCast(rA,&m));
+    PetscCall(PetscBLASIntCast(cB,&n));
+    PetscCall(PetscBLASIntCast(cA,&k));
     PetscCallBLAS("BLASgemm",BLASgemm_(qA,qB,&m,&n,&k,&a,(PetscScalar*)A,&ldA,(PetscScalar*)B,&ldB,&b,C,&ldC));
   }
 
@@ -502,7 +537,7 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    DSPseudoOrthogonalize - Orthogonalize the columns of a matrix with Modified
    Gram-Schmidt in an indefinite inner product space defined by a signature.
 
@@ -525,7 +560,7 @@ static PetscErrorCode SlepcMatDenseMult(PetscScalar *C,PetscInt _ldC,PetscScalar
 
 .seealso: DSOrthogonalize()
 @*/
-PetscErrorCode DSPseudoOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscReal *s,PetscInt *lindcols,PetscReal *ns)
+PetscErrorCode DSPseudoOrthogonalize(DS ds,DSMatType mat,PetscInt cols,PetscReal s[],PetscInt *lindcols,PetscReal ns[])
 {
   PetscInt       i,j,k,l,n,ld;
   PetscBLASInt   info,one=1,zero=0,rA_,ld_;
