@@ -30,24 +30,28 @@ static PetscErrorCode RGRingSetParameters_Ring(RG rg,PetscScalar center,PetscRea
 
   PetscFunctionBegin;
   ctx->center = center;
-  if (radius == (PetscReal)PETSC_DEFAULT) {
+  if (radius == (PetscReal)PETSC_DETERMINE) {
     ctx->radius = 1.0;
-  } else {
+  } else if (radius != (PetscReal)PETSC_CURRENT) {
     PetscCheck(radius>0.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The radius argument must be > 0.0");
     ctx->radius = radius;
   }
-  PetscCheck(vscale>0.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The vscale argument must be > 0.0");
-  ctx->vscale = vscale;
-  if (start_ang == (PetscReal)PETSC_DEFAULT) {
+  if (vscale == (PetscReal)PETSC_DETERMINE) {
+    ctx->vscale = 1.0;
+  } else if (vscale != (PetscReal)PETSC_CURRENT) {
+    PetscCheck(vscale>0.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The vscale argument must be > 0.0");
+    ctx->vscale = vscale;
+  }
+  if (start_ang == (PetscReal)PETSC_DETERMINE) {
     ctx->start_ang = 0.0;
-  } else {
+  } else if (start_ang != (PetscReal)PETSC_CURRENT) {
     PetscCheck(start_ang>=0.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The right-hand side angle argument must be >= 0.0");
     PetscCheck(start_ang<=1.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The right-hand side angle argument must be <= 1.0");
     ctx->start_ang = start_ang;
   }
-  if (end_ang == (PetscReal)PETSC_DEFAULT) {
+  if (end_ang == (PetscReal)PETSC_DETERMINE) {
     ctx->end_ang = 1.0;
-  } else {
+  } else if (end_ang != (PetscReal)PETSC_CURRENT) {
     PetscCheck(end_ang>=0.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The left-hand side angle argument must be >= 0.0");
     PetscCheck(end_ang<=1.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The left-hand side angle argument must be <= 1.0");
     ctx->end_ang = end_ang;
@@ -55,9 +59,9 @@ static PetscErrorCode RGRingSetParameters_Ring(RG rg,PetscScalar center,PetscRea
 #if !defined(PETSC_USE_COMPLEX)
   PetscCheck(ctx->start_ang+ctx->end_ang==1.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_WRONG,"In real scalars the region must be symmetric wrt real axis");
 #endif
-  if (width == (PetscReal)PETSC_DEFAULT) {
+  if (width == (PetscReal)PETSC_DETERMINE) {
     ctx->width = 0.1;
-  } else {
+  } else if (width != (PetscReal)PETSC_CURRENT) {
     PetscCheck(width>0.0,PetscObjectComm((PetscObject)rg),PETSC_ERR_ARG_OUTOFRANGE,"The width argument must be > 0.0");
     ctx->width = width;
   }
@@ -106,6 +110,10 @@ static PetscErrorCode RGRingSetParameters_Ring(RG rg,PetscScalar center,PetscRea
    and the start and end angles must be such that the region is symmetric with
    respect to the real axis.
 
+   For all arguments except center, you can use PETSC_CURRENT to keep the current
+   value, and PETSC_DETERMINE to set them to a default value (1 for radius, vscale,
+   end_ang, 0 for start_and, and 0.1 for width).
+
    Level: advanced
 
 .seealso: RGRingGetParameters()
@@ -166,7 +174,7 @@ PetscErrorCode RGRingGetParameters(RG rg,PetscScalar *center,PetscReal *radius,P
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGView_Ring(RG rg,PetscViewer viewer)
+static PetscErrorCode RGView_Ring(RG rg,PetscViewer viewer)
 {
   RG_RING        *ctx = (RG_RING*)rg->data;
   int            winw,winh;
@@ -229,7 +237,7 @@ PetscErrorCode RGView_Ring(RG rg,PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGIsTrivial_Ring(RG rg,PetscBool *trivial)
+static PetscErrorCode RGIsTrivial_Ring(RG rg,PetscBool *trivial)
 {
   RG_RING *ctx = (RG_RING*)rg->data;
 
@@ -239,7 +247,7 @@ PetscErrorCode RGIsTrivial_Ring(RG rg,PetscBool *trivial)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGComputeContour_Ring(RG rg,PetscInt n,PetscScalar *cr,PetscScalar *ci)
+static PetscErrorCode RGComputeContour_Ring(RG rg,PetscInt n,PetscScalar *cr,PetscScalar *ci)
 {
   RG_RING   *ctx = (RG_RING*)rg->data;
   PetscReal theta,start_ang;
@@ -269,7 +277,7 @@ PetscErrorCode RGComputeContour_Ring(RG rg,PetscInt n,PetscScalar *cr,PetscScala
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGComputeBoundingBox_Ring(RG rg,PetscReal *a,PetscReal *b,PetscReal *c,PetscReal *d)
+static PetscErrorCode RGComputeBoundingBox_Ring(RG rg,PetscReal *a,PetscReal *b,PetscReal *c,PetscReal *d)
 {
   RG_RING *ctx = (RG_RING*)rg->data;
 
@@ -282,7 +290,7 @@ PetscErrorCode RGComputeBoundingBox_Ring(RG rg,PetscReal *a,PetscReal *b,PetscRe
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGComputeQuadrature_Ring(RG rg,RGQuadRule quad,PetscInt n,PetscScalar *z,PetscScalar *zn,PetscScalar *w)
+static PetscErrorCode RGComputeQuadrature_Ring(RG rg,RGQuadRule quad,PetscInt n,PetscScalar *z,PetscScalar *zn,PetscScalar *w)
 {
   RG_RING     *ctx = (RG_RING*)rg->data;
   PetscReal   max_w=0.0;
@@ -319,7 +327,7 @@ PetscErrorCode RGComputeQuadrature_Ring(RG rg,RGQuadRule quad,PetscInt n,PetscSc
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGCheckInside_Ring(RG rg,PetscReal px,PetscReal py,PetscInt *inside)
+static PetscErrorCode RGCheckInside_Ring(RG rg,PetscReal px,PetscReal py,PetscInt *inside)
 {
   RG_RING   *ctx = (RG_RING*)rg->data;
   PetscReal dx,dy,r;
@@ -371,7 +379,7 @@ PetscErrorCode RGCheckInside_Ring(RG rg,PetscReal px,PetscReal py,PetscInt *insi
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGIsAxisymmetric_Ring(RG rg,PetscBool vertical,PetscBool *symm)
+static PetscErrorCode RGIsAxisymmetric_Ring(RG rg,PetscBool vertical,PetscBool *symm)
 {
   RG_RING *ctx = (RG_RING*)rg->data;
 
@@ -381,7 +389,7 @@ PetscErrorCode RGIsAxisymmetric_Ring(RG rg,PetscBool vertical,PetscBool *symm)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGSetFromOptions_Ring(RG rg,PetscOptionItems *PetscOptionsObject)
+static PetscErrorCode RGSetFromOptions_Ring(RG rg,PetscOptionItems PetscOptionsObject)
 {
   PetscScalar    s;
   PetscReal      r1,r2,r3,r4,r5;
@@ -403,7 +411,7 @@ PetscErrorCode RGSetFromOptions_Ring(RG rg,PetscOptionItems *PetscOptionsObject)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGDestroy_Ring(RG rg)
+static PetscErrorCode RGDestroy_Ring(RG rg)
 {
   PetscFunctionBegin;
   PetscCall(PetscFree(rg->data));
